# DO NOT EDIT!
# This file was automatically generated by "generate_id_constants_from_stableid.py"

import enum

class UpgradeId(enum.Enum):
    NULL = 0
    CARRIERLAUNCHSPEEDUPGRADE = 1
    GLIALRECONSTITUTION = 2
    TUNNELINGCLAWS = 3
    CHITINOUSPLATING = 4
    HISECAUTOTRACKING = 5
    TERRANBUILDINGARMOR = 6
    TERRANINFANTRYWEAPONSLEVEL1 = 7
    TERRANINFANTRYWEAPONSLEVEL2 = 8
    TERRANINFANTRYWEAPONSLEVEL3 = 9
    NEOSTEELFRAME = 10
    TERRANINFANTRYARMORSLEVEL1 = 11
    TERRANINFANTRYARMORSLEVEL2 = 12
    TERRANINFANTRYARMORSLEVEL3 = 13
    REAPERSPEED = 14
    STIMPACK = 15
    SHIELDWALL = 16
    PUNISHERGRENADES = 17
    SIEGETECH = 18
    HIGHCAPACITYBARRELS = 19
    BANSHEECLOAK = 20
    MEDIVACCADUCEUSREACTOR = 21
    RAVENCORVIDREACTOR = 22
    HUNTERSEEKER = 23
    DURABLEMATERIALS = 24
    PERSONALCLOAKING = 25
    GHOSTMOEBIUSREACTOR = 26
    TERRANVEHICLEARMORSLEVEL1 = 27
    TERRANVEHICLEARMORSLEVEL2 = 28
    TERRANVEHICLEARMORSLEVEL3 = 29
    TERRANVEHICLEWEAPONSLEVEL1 = 30
    TERRANVEHICLEWEAPONSLEVEL2 = 31
    TERRANVEHICLEWEAPONSLEVEL3 = 32
    TERRANSHIPARMORSLEVEL1 = 33
    TERRANSHIPARMORSLEVEL2 = 34
    TERRANSHIPARMORSLEVEL3 = 35
    TERRANSHIPWEAPONSLEVEL1 = 36
    TERRANSHIPWEAPONSLEVEL2 = 37
    TERRANSHIPWEAPONSLEVEL3 = 38
    PROTOSSGROUNDWEAPONSLEVEL1 = 39
    PROTOSSGROUNDWEAPONSLEVEL2 = 40
    PROTOSSGROUNDWEAPONSLEVEL3 = 41
    PROTOSSGROUNDARMORSLEVEL1 = 42
    PROTOSSGROUNDARMORSLEVEL2 = 43
    PROTOSSGROUNDARMORSLEVEL3 = 44
    PROTOSSSHIELDSLEVEL1 = 45
    PROTOSSSHIELDSLEVEL2 = 46
    PROTOSSSHIELDSLEVEL3 = 47
    OBSERVERGRAVITICBOOSTER = 48
    GRAVITICDRIVE = 49
    EXTENDEDTHERMALLANCE = 50
    HIGHTEMPLARKHAYDARINAMULET = 51
    PSISTORMTECH = 52
    ZERGMELEEWEAPONSLEVEL1 = 53
    ZERGMELEEWEAPONSLEVEL2 = 54
    ZERGMELEEWEAPONSLEVEL3 = 55
    ZERGGROUNDARMORSLEVEL1 = 56
    ZERGGROUNDARMORSLEVEL2 = 57
    ZERGGROUNDARMORSLEVEL3 = 58
    ZERGMISSILEWEAPONSLEVEL1 = 59
    ZERGMISSILEWEAPONSLEVEL2 = 60
    ZERGMISSILEWEAPONSLEVEL3 = 61
    OVERLORDSPEED = 62
    OVERLORDTRANSPORT = 63
    BURROW = 64
    ZERGLINGATTACKSPEED = 65
    ZERGLINGMOVEMENTSPEED = 66
    HYDRALISKSPEED = 67
    ZERGFLYERWEAPONSLEVEL1 = 68
    ZERGFLYERWEAPONSLEVEL2 = 69
    ZERGFLYERWEAPONSLEVEL3 = 70
    ZERGFLYERARMORSLEVEL1 = 71
    ZERGFLYERARMORSLEVEL2 = 72
    ZERGFLYERARMORSLEVEL3 = 73
    INFESTORENERGYUPGRADE = 74
    CENTRIFICALHOOKS = 75
    BATTLECRUISERENABLESPECIALIZATIONS = 76
    BATTLECRUISERBEHEMOTHREACTOR = 77
    PROTOSSAIRWEAPONSLEVEL1 = 78
    PROTOSSAIRWEAPONSLEVEL2 = 79
    PROTOSSAIRWEAPONSLEVEL3 = 80
    PROTOSSAIRARMORSLEVEL1 = 81
    PROTOSSAIRARMORSLEVEL2 = 82
    PROTOSSAIRARMORSLEVEL3 = 83
    WARPGATERESEARCH = 84
    HALTECH = 85
    CHARGE = 86
    BLINKTECH = 87
    ANABOLICSYNTHESIS = 88
    OBVERSEINCUBATION = 89
    VIKINGJOTUNBOOSTERS = 90
    ORGANICCARAPACE = 91
    INFESTORPERISTALSIS = 92
    ABDOMINALFORTITUDE = 93
    HYDRALISKSPEEDUPGRADE = 94
    BANELINGBURROWMOVE = 95
    COMBATDRUGS = 96
    STRIKECANNONS = 97
    TRANSFORMATIONSERVOS = 98
    PHOENIXRANGEUPGRADE = 99
    TEMPESTRANGEUPGRADE = 100
    NEURALPARASITE = 101
    LOCUSTLIFETIMEINCREASE = 102
    ULTRALISKBURROWCHARGEUPGRADE = 103
    ORACLEENERGYUPGRADE = 104
    RESTORESHIELDS = 105
    PROTOSSHEROSHIPWEAPON = 106
    PROTOSSHEROSHIPDETECTOR = 107
    PROTOSSHEROSHIPSPELL = 108
    REAPERJUMP = 109
    INCREASEDRANGE = 110
    ZERGBURROWMOVE = 111
    ANIONPULSECRYSTALS = 112
    TERRANVEHICLEANDSHIPWEAPONSLEVEL1 = 113
    TERRANVEHICLEANDSHIPWEAPONSLEVEL2 = 114
    TERRANVEHICLEANDSHIPWEAPONSLEVEL3 = 115
    TERRANVEHICLEANDSHIPARMORSLEVEL1 = 116
    TERRANVEHICLEANDSHIPARMORSLEVEL2 = 117
    TERRANVEHICLEANDSHIPARMORSLEVEL3 = 118
    FLYINGLOCUSTS = 119
    ROACHSUPPLY = 120
    IMMORTALREVIVE = 121
    DRILLCLAWS = 122
    CYCLONELOCKONRANGEUPGRADE = 123
    CYCLONEAIRUPGRADE = 124
    LIBERATORMORPH = 125
    ADEPTSHIELDUPGRADE = 126
    LURKERRANGE = 127
    IMMORTALBARRIER = 128
    ADEPTKILLBOUNCE = 129
    ADEPTPIERCINGATTACK = 130
    CINEMATICMODE = 131
    CURSORDEBUG = 132
    MAGFIELDLAUNCHERS = 133
    EVOLVEGROOVEDSPINES = 134
    EVOLVEMUSCULARAUGMENTS = 135
    BANSHEESPEED = 136
    MEDIVACRAPIDDEPLOYMENT = 137
    RAVENRECALIBRATEDEXPLOSIVES = 138
    MEDIVACINCREASESPEEDBOOST = 139
    LIBERATORAGRANGEUPGRADE = 140
    DARKTEMPLARBLINKUPGRADE = 141
    RAVAGERRANGE = 142
    RAVENDAMAGEUPGRADE = 143
    CYCLONELOCKONDAMAGEUPGRADE = 144
    ARESCLASSWEAPONSSYSTEMVIKING = 145
    AUTOHARVESTER = 146
    HYBRIDCPLASMAUPGRADEHARD = 147
    HYBRIDCPLASMAUPGRADEINSANE = 148
    INTERCEPTORLIMIT4 = 149
    INTERCEPTORLIMIT6 = 150
    _330MMBARRAGECANNONS = 151
    NOTPOSSIBLESIEGEMODE = 152
    NEOSTEELFRAME_2 = 153
    NEOSTEELANDSHRIKETURRETICONUPGRADE = 154
    OCULARIMPLANTS = 155
    CROSSSPECTRUMDAMPENERS = 156
    ORBITALSTRIKE = 157
    CLUSTERBOMB = 158
    SHAPEDHULL = 159
    SPECTRETOOLTIPUPGRADE = 160
    ULTRACAPACITORS = 161
    VANADIUMPLATING = 162
    COMMANDCENTERREACTOR = 163
    REGENERATIVEBIOSTEEL = 164
    CELLULARREACTORS = 165
    BANSHEECLOAKEDDAMAGE = 166
    DISTORTIONBLASTERS = 167
    EMPTOWER = 168
    SUPPLYDEPOTDROP = 169
    HIVEMINDEMULATOR = 170
    FORTIFIEDBUNKERCARAPACE = 171
    PREDATOR = 172
    SCIENCEVESSEL = 173
    DUALFUSIONWELDERS = 174
    ADVANCEDCONSTRUCTION = 175
    ADVANCEDMEDICTRAINING = 176
    PROJECTILEACCELERATORS = 177
    REINFORCEDSUPERSTRUCTURE = 178
    MULE = 179
    ORBITALRELAY = 180
    RAZORWIRE = 181
    ADVANCEDHEALINGAI = 182
    TWINLINKEDFLAMETHROWERS = 183
    NANOCONSTRUCTOR = 184
    CERBERUSMINES = 185
    HYPERFLUXOR = 186
    TRILITHIUMPOWERCELLS = 187
    PERMANENTCLOAKGHOST = 188
    PERMANENTCLOAKSPECTRE = 189
    ULTRASONICPULSE = 190
    SURVIVALPODS = 191
    ENERGYSTORAGE = 192
    FULLBORECANISTERAMMO = 193
    CAMPAIGNJOTUNBOOSTERS = 194
    MICROFILTERING = 195
    PARTICLECANNONAIR = 196
    VULTUREAUTOREPAIR = 197
    PSIDISRUPTOR = 198
    SCIENCEVESSELENERGYMANIPULATION = 199
    SCIENCEVESSELPLASMAWEAPONRY = 200
    SHOWGATLINGGUN = 201
    TECHREACTOR = 202
    TECHREACTORAI = 203
    TERRANDEFENSERANGEBONUS = 204
    X88TNAPALMUPGRADE = 205
    HURRICANEMISSILES = 206
    MECHANICALREBIRTH = 207
    MARINESTIMPACK = 208
    DARKTEMPLARTACTICS = 209
    CLUSTERWARHEADS = 210
    CLOAKDISTORTIONFIELD = 211
    DEVASTATORMISSILES = 212
    DISTORTIONTHRUSTERS = 213
    DYNAMICPOWERROUTING = 214
    IMPALERROUNDS = 215
    KINETICFIELDS = 216
    BURSTCAPACITORS = 217
    HAILSTORMMISSILEPODS = 218
    RAPIDDEPLOYMENT = 219
    REAPERSTIMPACK = 220
    REAPERD8CHARGE = 221
    TYCHUS05BATTLECRUISERPENETRATION = 222
    VIRALPLASMA = 223
    FIREBATJUGGERNAUTPLATING = 224
    MULTILOCKTARGETINGSYSTEMS = 225
    TURBOCHARGEDENGINES = 226
    DISTORTIONSENSORS = 227
    INFERNALPREIGNITERS = 228
    HELLIONCAMPAIGNINFERNALPREIGNITER = 229
    NAPALMFUELTANKS = 230
    AUXILIARYMEDBOTS = 231
    JUGGERNAUTPLATING = 232
    MARAUDERLIFEBOOST = 233
    COMBATSHIELD = 234
    REAPERU238ROUNDS = 235
    MAELSTROMROUNDS = 236
    SIEGETANKSHAPEDBLAST = 237
    TUNGSTENSPIKES = 238
    BEARCLAWNOZZLES = 239
    NANOBOTINJECTORS = 240
    STABILIZERMEDPACKS = 241
    HALOROCKETS = 242
    SCAVENGINGSYSTEMS = 243
    EXTRAMINES = 244
    ARESCLASSWEAPONSSYSTEM = 245
    WHITENAPALM = 246
    VIRALMUNITIONS = 247
    JACKHAMMERCONCUSSIONGRENADES = 248
    FIRESUPPRESSIONSYSTEMS = 249
    FLARERESEARCH = 250
    MODULARCONSTRUCTION = 251
    EXPANDEDHULL = 252
    SHRIKETURRET = 253
    MICROFUSIONREACTORS = 254
    WRAITHCLOAK = 255
    SINGULARITYCHARGE = 256
    GRAVITICTHRUSTERS = 257
    YAMATOCANNON = 258
    DEFENSIVEMATRIX = 259
    DARKPROTOSS = 260
    TERRANINFANTRYWEAPONSULTRACAPACITORSLEVEL1 = 261
    TERRANINFANTRYWEAPONSULTRACAPACITORSLEVEL2 = 262
    TERRANINFANTRYWEAPONSULTRACAPACITORSLEVEL3 = 263
    TERRANINFANTRYARMORSVANADIUMPLATINGLEVEL1 = 264
    TERRANINFANTRYARMORSVANADIUMPLATINGLEVEL2 = 265
    TERRANINFANTRYARMORSVANADIUMPLATINGLEVEL3 = 266
    TERRANVEHICLEWEAPONSULTRACAPACITORSLEVEL1 = 267
    TERRANVEHICLEWEAPONSULTRACAPACITORSLEVEL2 = 268
    TERRANVEHICLEWEAPONSULTRACAPACITORSLEVEL3 = 269
    TERRANVEHICLEARMORSVANADIUMPLATINGLEVEL1 = 270
    TERRANVEHICLEARMORSVANADIUMPLATINGLEVEL2 = 271
    TERRANVEHICLEARMORSVANADIUMPLATINGLEVEL3 = 272
    TERRANSHIPWEAPONSULTRACAPACITORSLEVEL1 = 273
    TERRANSHIPWEAPONSULTRACAPACITORSLEVEL2 = 274
    TERRANSHIPWEAPONSULTRACAPACITORSLEVEL3 = 275
    TERRANSHIPARMORSVANADIUMPLATINGLEVEL1 = 276
    TERRANSHIPARMORSVANADIUMPLATINGLEVEL2 = 277
    TERRANSHIPARMORSVANADIUMPLATINGLEVEL3 = 278
    HIREKELMORIANMINERSPH = 279
    HIREDEVILDOGSPH = 280
    HIRESPARTANCOMPANYPH = 281
    HIREHAMMERSECURITIESPH = 282
    HIRESIEGEBREAKERSPH = 283
    HIREHELSANGELSPH = 284
    HIREDUSKWINGPH = 285
    HIREDUKESREVENGE = 286
    TOSHEASYMODE = 287
    VOIDRAYSPEEDUPGRADE = 288
    SMARTSERVOS = 289
    ARMORPIERCINGROCKETS = 290
    CYCLONERAPIDFIRELAUNCHERS = 291
    RAVENENHANCEDMUNITIONS = 292
    DIGGINGCLAWS = 293
    CARRIERCARRIERCAPACITY = 294
    CARRIERLEASHRANGEUPGRADE = 295

for item in UpgradeId:
    assert not item.name in globals()
    globals()[item.name] = item
